'Solarbotics combination Minisumo/Linefollower program
'Revision Date: Mar. 14th '07
'Program written for modified 28x PICaxe Stack on a modified BS2 Brain board
'Programmer: Dan Gates

'Behavior as follows:

'Upon powerup the processor reads the line sensor inputs, if the surface is black the
'program kicks into sumo mode if the surface is white the program will go to
'linefollower mode.

'Sumo mode:
'sumo will wait for 5 seconds before moving
'at each passing second another LED will light from left to right.
'Once 5 seconds is up the minisumo will turn off all LED's and go strait forward.
'If a line is detected the minisumo will back up and turn away from the detected side,
'then go strait forward again.
'If an opponent is detected the minisumo will disable the motor on that side
'to turn towards the opponent.
'If both IR detector activate then the minsumo will go strait forward.
'This follows the same behavior as the discreet brain.

'Linefollower mode:
'Line follower has no startup delay
'If either one of the sensors on either side of the line detects black
'it will make a soft turn.
'If either one of the sensors on the outer edge detects a line it will make
'a longer, sharper turn.
'The LED's indicate what the line sensors are seeing.
'Thus far the center line sensor, if installed, only controls the third LED.


'----------------------------- Pin Allocations -----------------------------

'Line sensors - only using outer right and left for minisumo
'Value is 1 seeing black and 0 seeing white
symbol Right_Line = 		pin1
symbol Center_Right_Line =  	pin2
symbol Center_Line =		pin3
symbol Center_Left_Line =	pin4
symbol Left_Line =		pin5

'IR opponent detect not used in Line follower
symbol Right_IR =			pin6      'Right IR detector active LOW
symbol Left_IR =			pin7      'Left IR detector active LOW

'Motor control
symbol Enable_Right =       0        'Enable high, disable low
symbol Enable_Left =        1        'Enable high, disable low
symbol Right_Direction =    2        '1 for forward and 0 for backward
symbol Left_Direction =     3        '1 for forward and 0 for backward

let dirsc  =           %00000001

'LED indicatiors, Active high
symbol LED1 =               4
symbol LED2 =               5
symbol LED3 =               6
symbol LED4 =               7
'LED5 is on portc 0 which cannot be assigned to a symbol


'----------------------------- Variable Definitions ----------------------

symbol Temp   =     W1   'General temp variable for timers, counters ect.


'------------------------------ Minisumo/Linefollower switch ---------------------

'CHECK:
  IF Right_Line = 1 AND Left_Line = 1 THEN START          'If on black surface be a Sumo
  IF Right_Line = 0 AND Left_Line = 0 THEN LINEFOLLOWER   'If on white surface be a Linefollower

'----------------------------- Minisumo program start -----------------------

START:
  FOR Temp = 4 TO 7  	'LED sequence
  HIGH TEMP           	'Light up the pretty LEDs one at a time
  PAUSE 1000          	'Pause for one second
  NEXT			'Loop back till first 4 LEDs are lit
  HIGH portc 0		'Light the fifth LED
  PAUSE 1000		'Continue delay to equal a total of 5 seconds

  LOW 4             	'Turn off LED's
  LOW 5
  LOW 6
  LOW 7
  LOW portc 0

SUMO:
  HIGH Enable_Right 	'Added so sumo doesn't stall if it see's something outside the ring
  HIGH Enable_Left  	'Added so sumo doesn't stall if it see's something outside the ring
  IF Right_Line = 0 THEN Back_Right	'If right sensor sees edge, jump to back and right manuvor
  IF Left_Line = 0 THEN Back_Left 	'If left sensor sees edge, jump to back and left manuvor
  GOSUB STRAIGHT		'If all is clear, go straight forward
  GOSUB SEEK    		'Take a look around, if something is there then turn towards it
  GOTO SUMO

STRAIGHT:                  'Go straight forward subroutine
  HIGH Right_Direction
  HIGH Left_Direction
  HIGH Enable_Right
  HIGH Enable_Left
RETURN

Back_Right:
  HIGH portc 0                    'Indicate Line detect
  LOW Right_Direction             'Reverse Right motor
  LOW Left_Direction              'Reverse left motor
  PAUSE 200                        'For 10 milliseconds
  HIGH Right_Direction               'Shut off right motor
  PAUSE 500
  LOW portc 0			    'Turn off indicator
GOTO SUMO

Back_Left:
  HIGH LED1                       'Indicate Line detect
  LOW Left_Direction              'Reverse Left motor
  LOW Right_Direction             'Reverse right motor
  PAUSE 200                        'For 10 milliseconds
  FOR temp = 0 TO 550		    'Loop for awhile
  HIGH Left_Direction                'Shut off left motor
  PAUSE 500
  LOW LED1 				    'Turn off indicator
GOTO SUMO

SEEK:
  IF Right_IR = Left_IR THEN SUMO  	  'IF both detect then go forward
  IF Right_IR = 0 THEN Turn_Right     'If detect on right then turn right
  IF Left_IR = 0 THEN Turn_Left       'If detect on left then turn left
RETURN

Turn_Right:
  LOW Enable_Right               'Shut off right motor to turn right
  PAUSE 100
GOTO SUMO

Turn_Left:
  LOW Enable_Left                'Shut off left motor to turn left
  PAUSE 100
GOTO SUMO

'-------------------------------- Line Follower Code -----------------------------------

LINEFOLLOWER:
  GOSUB GO_FORWARD 'Start going forward

  LOW LED1			'Make sure all LEDs get turned off after each use
  LOW LED2
  LOW LED3
  LOW LED4
  LOW portc 0

  IF Right_Line = 1 THEN HARD_Right 	'If outer right sensor sees line then hard turn right
  IF Left_Line = 1 THEN HARD_Left   	'If outer left sensor sees line then hard turn left

  IF Center_Right_Line = 1 THEN GO_Right 	'If inner right sensor sees line then soft turn right
  IF Center_Left_Line = 1 THEN GO_Left   	'If inner left sensor sees line then soft turn left
  IF Center_Line  = 1 THEN Center	      'If center sensor sees line then light LED3 and go forward

GOTO LINEFOLLOWER

GO_FORWARD:
  HIGH Right_Direction		 'Turn on both motors and go forward
  HIGH Left_Direction
  HIGH Enable_Right
  HIGH Enable_Left
RETURN

GO_Right:                      'Soft turn right
  HIGH LED4				 'Turn on LED4
  LOW Right_Direction          'Reverse right motor
  PAUSE 10                     'for 10 ms will quickly slow but not stop the motor
  LOW Enable_Right
  PAUSE 15
GOTO LINEFOLLOWER

HARD_Right:				 'Hard turn right
  HIGH portc 0                  'Turn on LED5
  LOW Right_Direction          'Reverse Right motor
  PAUSE 150                    'Give short time to make it happen
GOTO LINEFOLLOWER

GO_Left:                       'Soft turn left
  HIGH LED2				 'Turn on LED2
  LOW Left_Direction           'Reverse Left motor
  PAUSE 10                     'for 10 ms will quickly slow but not stop the motor
  LOW Enable_Left
  PAUSE 15
GOTO LINEFOLLOWER

HARD_Left:				 'Hard turn left
  HIGH LED1                    'Turn on LED1
  LOW Left_Direction           'Reverse Left motor
  PAUSE 150                    'Give short time to make it happen
GOTO LINEFOLLOWER

Center:
  High LED3
GOTO LINEFOLLOWER


